/**
 * 
 */
package jalview.gui;

import java.awt.Container;
import java.util.BitSet;

import javax.swing.JPanel;

import jalview.bin.Cache;
import jalview.datamodel.PDBEntry;
import jalview.datamodel.SequenceI;

import org.jmol.api.JmolAppConsoleInterface;
import org.jmol.api.JmolViewer;
import org.jmol.popup.JmolPopup;
import org.openscience.jmol.app.jmolpanel.AppConsole;

public class AppJmolBinding extends jalview.ext.jmol.JalviewJmolBinding
{

  /**
   * 
   */
  private AppJmol appJmolWindow;

  public AppJmolBinding(AppJmol appJmol, PDBEntry[] pdbentry,
          SequenceI[][] sequenceIs, String[][] chains, String protocol)
  {
    super(pdbentry, sequenceIs, chains, protocol);
    appJmolWindow = appJmol;
  }

  FeatureRenderer fr = null;

  @Override
  public jalview.api.FeatureRenderer getFeatureRenderer()
  {
    if (appJmolWindow.ap.av.showSequenceFeatures)
    {
      if (fr == null)
      {
        fr = new FeatureRenderer(appJmolWindow.ap);
      }

      fr.transferSettings(appJmolWindow.ap.seqPanel.seqCanvas
              .getFeatureRenderer());
    }

    return fr;
  }

  @Override
  public jalview.api.SequenceRenderer getSequenceRenderer()
  {
    return new SequenceRenderer(appJmolWindow.ap.av);
  }

  public void sendConsoleEcho(String strEcho)
  {
    if (console != null)
    {
      console.sendConsoleEcho(strEcho);
    }
  }

  public void sendConsoleMessage(String strStatus)
  {
    if (console != null && strStatus != null)
    // && !strStatus.equals("Script completed"))
    // should we squash the script completed string ?
    {
      console.sendConsoleMessage(strStatus);
    }
  }

  @Override
  public void showUrl(String url, String target)
  {
    try
    {
      jalview.util.BrowserLauncher.openURL(url);
    } catch (Exception e)
    {
      Cache.log.error("Failed to launch Jmol-associated url " + url, e);
      // TODO: 2.6 : warn user if browser was not configured.
    }
  }

  @Override
  public void refreshGUI()
  {
    // appJmolWindow.repaint();
    javax.swing.SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        appJmolWindow.updateTitleAndMenus();
        appJmolWindow.revalidate();
      }});
  }

  public void updateColours(Object source)
  {
    AlignmentPanel ap = (AlignmentPanel) source;
    if (appJmolWindow.ap.alignFrame.getCurrentView() != ap.av)
      return;

    colourBySequence(ap.av.getShowSequenceFeatures(), ap.av.alignment);
  }

  public void notifyScriptTermination(String strStatus, int msWalltime)
  {
    // todo - script termination doesn't happen ?
    //if (console != null)
      //console.notifyScriptTermination(strStatus,
      //        msWalltime);
  }

  public void showUrl(String url)
  {
    showUrl(url, "jmol");
  }

  public void newJmolPopup(boolean translateLocale, String menuName,
          boolean asPopup)
  {

    jmolpopup = JmolPopup.newJmolPopup(viewer, translateLocale, menuName,
            asPopup);
  }

  public void selectionChanged(BitSet arg0)
  {
    // TODO Auto-generated method stub

  }

  public void refreshPdbEntries()
  {
    // TODO Auto-generated method stub

  }

  public void showConsole(boolean b) {
    appJmolWindow.showConsole(b);
  }

  /**
   * add the given sequences to the mapping scope for the given pdb file handle
   * 
   * @param pdbFile
   *          - pdbFile identifier
   * @param seq
   *          - set of sequences it can be mapped to
   */
  public void addSequenceForStructFile(String pdbFile, SequenceI[] seq)
  {
    for (int pe = 0; pe < pdbentry.length; pe++)
    {
      if (pdbentry[pe].getFile().equals(pdbFile))
      {
        addSequence(pe, seq);
      }
    }
  }

  @Override
  protected JmolAppConsoleInterface createJmolConsole(JmolViewer viewer2,
          Container consolePanel, String buttonsToShow)
  {
    return new AppConsole(viewer, null, consolePanel, buttonsToShow);
  }
}